package com.uas.customsdeclaration.business.common.service.impl;

import com.uas.account.sso.integration.entity.UserAccount;
import com.uas.account.sso.integration.util.SSO;
import com.uas.customsdeclaration.business.common.dao.AddressDao;
import com.uas.customsdeclaration.business.common.model.Address;
import com.uas.customsdeclaration.business.common.service.AddressService;
import com.uas.customsdeclaration.business.common.type.AddressType;
import com.uas.customsdeclaration.core.IntegerConstant;
import com.uas.customsdeclaration.util.CodeType;
import com.uas.customsdeclaration.util.ResultMap;
import com.uas.customsdeclaration.util.StringUtils;
import com.uas.platform.core.model.PageInfo;
import com.uas.platform.core.model.PageParams;
import com.uas.platform.core.persistence.criteria.CriterionExpression;
import com.uas.platform.core.persistence.criteria.SimpleExpression;
import com.uas.platform.core.util.serializer.FlexJsonUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.persistence.EntityManager;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.Date;
import java.util.List;

/**
 * 地址管理的实现层
 *
 * @author yuj 2018-04-19 15:09
 */
@Service
public class AddressServiceImpl implements AddressService {

    private final AddressDao addressDao;

    private final EntityManager entityManager;

    @Autowired
    public AddressServiceImpl(AddressDao addressDao, EntityManager entityManager) {
        this.addressDao = addressDao;
        this.entityManager = entityManager;
    }


    /**
     * 保存地址信息
     *
     * @param params    需要保存的地址信息
     * @param type      地址类型
     * @param isDefault 是否是默认地址信息
     * @return ResultMap 返回保存成功的地址信息
     */
    @Override
    public ResultMap save(String params, Integer type, Boolean isDefault) {
        if (StringUtils.isEmpty(params) || type == null || isDefault == null) {
            return new ResultMap(CodeType.DEFICIENCY_INFO, "地址信息、类型或默认地址信息缺失");
        }
        AddressType addressType = AddressType.valueof(type);
        if (addressType == null) {
            return new ResultMap(CodeType.DEFICIENCY_INFO, "地址类型不正确");
        }
        Address address = null;
        try {
            address = FlexJsonUtils.fromJson(params, Address.class);
        } catch (Exception e) {
            return new ResultMap(CodeType.DEFICIENCY_INFO, "地址信息转化失败，传入的格式不对");
        }
        if (address == null) {
            return new ResultMap(CodeType.DEFICIENCY_INFO, "地址信息缺失，请填写信息，重新操作");
        }
        address.setType(addressType.getType());
        address.setTypePhrase(addressType.getPhrase());
        address.setCreateTime(new Date());
        UserAccount userAccount = SSO.get();
        address.setEnuu(userAccount.getSpaceUU());
        address.setCompany(userAccount.getSpaceName());
        address.setOperateUseruu(userAccount.getUserUU());
        address.setOperateUserName(userAccount.getVipName());

        Boolean info = address.validateInfo();
        if (!info) {
            return new ResultMap(CodeType.DEFICIENCY_INFO, "请完善地址信息");
        }
        setAddressNum(address, isDefault);
        Address address1 = addressDao.save(address);
        return ResultMap.success(address1);
    }

    /**
     * 设置地址信息的排序数
     *
     * @param address
     * @param isDefault
     */
    @Override
    public void setAddressNum(Address address, Boolean isDefault) {
        if (address == null) {
            return;
        } else {
            if (address.getEnuu() == null || address.getType() == null) {
                return ;
            }
            List<Address> addresses = addressDao.findAddressByEnuuAndTypeOrderNum(address.getEnuu(), address.getType());
            if (CollectionUtils.isEmpty(addresses)) {
                address.setNum(IntegerConstant.single);
            } else {
                if (isDefault) {
                    address.setNum(IntegerConstant.single);
                    int i = 2;
                    for (Address address1 : addresses) {
                        address1.setNum(i++);
                    }
                    addressDao.save(addresses);
                } else {
                    address.setNum(addresses.size() + 1);
                }

            }
        }


    }

    /**
     * 设置地址为默认地址
     *
     * @param id
     * @return
     */
    @Override
    public ResultMap setAddressDefault(Long id) {
        if (id == null) {
            return new ResultMap(CodeType.DEFICIENCY_INFO, "传入的主键信息为空");
        }
        Address address = addressDao.findOne(id);
        if (address == null) {
            return new ResultMap(CodeType.FAILURE, "找不到对应地址信息");
        }
        List<Address> addresses = findAddressByTypeAndAddress(address, AddressType.valueof(address.getType()));
        Integer i = 2;
        for (Address address1 : addresses) {
            if (address.getId().longValue() == address1.getId().longValue()) {
                address1.setNum(IntegerConstant.single);
            } else {
                address1.setNum(i++);
            }
        }
        Address addrs = null;
        List<Address> addressList = addressDao.save(addresses);
        for (Address address1 : addressList) {
            if (address1.getId().longValue() == id.longValue()) {
                addrs = address1;
            }
        }
        return ResultMap.success(addrs);
    }

    /**
     * 分页获取地址信息
     *
     * @param params 分页信息
     * @param type   地址类型
     * @return ResultMap
     */
    @Override
    public ResultMap findAddressByPage(PageParams params, Integer type) {
        if (type == null) {
            return new ResultMap(CodeType.DEFICIENCY_INFO, "地址类型为空");
        } else {
            AddressType addressType = AddressType.valueof(type);
            if (addressType == null) {
                return new ResultMap(CodeType.FAILURE, "传入的地址类型不对");
            }
        }
        UserAccount account = SSO.get();
        Long spaceUU = account.getSpaceUU();
        final PageInfo info = new PageInfo(params);
        info.sorting("num", Sort.Direction.ASC);
        SimpleExpression expression = new SimpleExpression("type", type, CriterionExpression.Operator.EQ);
        SimpleExpression expression1 = new SimpleExpression("enuu", spaceUU, CriterionExpression.Operator.EQ);
        if (type == AddressType.PERSONAL_COMPANY_ADDRESS.getType() || type == AddressType.DELIVERY_ADDRESS.getType()) {
            SimpleExpression expression2 = new SimpleExpression("operateUseruu", account.getUserUU(), CriterionExpression.Operator.EQ);
            info.expression(expression2);
        }
        info.expression(expression);
        info.expression(expression1);
        Page<Address> addressPage = addressDao.findAll(new Specification<Address>() {
            @Override
            public Predicate toPredicate(Root<Address> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
                query.where(info.getPredicates(root, query, cb));
                return null;
            }
        }, info);
        return ResultMap.success(addressPage);
    }

    /**
     * 根据地址信息和地址类型获取 个人或企业的地址信息
     *
     * @param address
     * @param type
     * @return
     */
    public List<Address> findAddressByTypeAndAddress(Address address, AddressType type) {
        if (type == null || address == null) {
            return null;
        } else {
            List<Address> addresses = null;
            if (type.getType() == AddressType.PERSONAL_COMPANY_ADDRESS.getType() || type.getType() == AddressType.DELIVER_ADDRESS.getType()) {
                addresses = addressDao.findAddressByEnuuAndOperateUseruuAndTypeOrderNum(address.getEnuu(), address.getOperateUseruu(), type.getType());
            } else {
                addresses = addressDao.findAddressByEnuuAndTypeOrderNum(address.getEnuu(), type.getType());
            }
            return addresses;
        }
    }

    /**
     * 更新保存之后的信息
     *
     * @param json 更新后的信息
     * @param isDefault 是否设置为默认地址
     * @return ResultMap
     */
    @Override
    public ResultMap updateAddress(String json, Boolean isDefault) {
        if (StringUtils.isEmpty(json)) {
            return new ResultMap(CodeType.FAILURE, "传入的信息为空");
        } else {
            Address address = FlexJsonUtils.fromJson(json, Address.class);
            Boolean completeInfo = address.validateInfo();
            if (!completeInfo) {
                return new ResultMap(CodeType.FAILURE, "请完善对应信息");
            }
            if (isDefault) {
                List<Address> addresses = addressDao.findAddressByEnuuAndTypeOrderNum(address.getEnuu(), address.getType());
                int i = 2;
                for (Address address1 : addresses) {
                    if (address1.getId().longValue() != address.getId().longValue()) {
                        address1.setNum(i++);
                    }
                }
                addressDao.save(addresses);
                address.setNum(IntegerConstant.single);
            }
            Address address1 = addressDao.save(address);
            return ResultMap.success(address1);
        }
    }

    /**
     * 根据id主键获取地址信息
     *
     * @param id
     * @return ResultMap
     */
    @Override
    public ResultMap getAddressById(Long id) {
        if (id == null) {
            return new ResultMap(CodeType.FAILURE, "信息唯一字段丢失，请重新上传");
        } else {
            Address address = addressDao.findOne(id);
            return ResultMap.success(address);
        }
    }


    /**
     * 删除指定的地址信息
     *
     * @param id 需要被删除的主键信息
     * @return ResultMap
     */
    @Transactional
    @Override
    public ResultMap deleteAddressById(Long id) {
        if (id == null) {
            return new ResultMap(CodeType.FAILURE, "请输入需要删除的信息");
        } else {
            Address address = addressDao.findOne(id);
            if (address == null) {
                return new ResultMap(CodeType.FAILURE, "找不到需要删除的信息");
            }
            AddressType addressType = AddressType.valueof(address.getType());
            UserAccount account = SSO.get();
            Long spaceUU = account.getSpaceUU();
            if (spaceUU == null || spaceUU.longValue() != address.getEnuu().longValue()) {
                return new ResultMap(CodeType.FAILURE, "您没有权限操作该信息");
            }
            if (addressType.getType() == AddressType.PERSONAL_COMPANY_ADDRESS.getType() || addressType.getType() == AddressType.DELIVER_ADDRESS.getType()) {
                if ((account.getUserUU().longValue() != address.getOperateUseruu())) {
                    return new ResultMap(CodeType.FAILURE, "您没有权限操作该信息");
                }
            }
            addressDao.delete(id);
            List<Address> addresses = findAddressByTypeAndAddress(address, addressType);
            int i = 1;
            for (Address address1 : addresses) {
                address1.setNum(i);
                i++;
            }
            addressDao.save(addresses);
            return ResultMap.success(address);
        }
    }
}
