package com.uas.customsdeclaration.business.customer.service.impl;

import com.uas.account.sso.integration.entity.UserAccount;
import com.uas.account.sso.integration.util.SSO;
import com.uas.customsdeclaration.business.customer.dao.CustomerDao;
import com.uas.customsdeclaration.business.customer.model.Customer;
import com.uas.customsdeclaration.business.customer.service.CustomerService;
import com.uas.customsdeclaration.util.CodeType;
import com.uas.customsdeclaration.util.ResultMap;
import com.uas.customsdeclaration.util.SSOUtil;
import com.uas.customsdeclaration.util.StringUtils;
import com.uas.platform.core.model.PageInfo;
import com.uas.platform.core.model.PageParams;
import com.uas.platform.core.util.serializer.FlexJsonUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 客户的实现层
 *
 * @author yuj 2018-03-19 15:57
 */
@Service
public class CustomerServiceImpl implements CustomerService {

    private final CustomerDao customerDao;

    private final SSOUtil ssoUtil;

    @Autowired
    public CustomerServiceImpl(CustomerDao customerDao, SSOUtil ssoUtil) {
        this.customerDao = customerDao;
        this.ssoUtil = ssoUtil;
    }

    /**
     * 根据企业UU获取客户信息
     *
     * @param enuu
     * @return
     */
    @Override
    public ResultMap findByEnuu(Long enuu) {
        if (enuu == null) {
            return new ResultMap(CodeType.FAILURE, "信息缺失");
        } else {
            Customer customer = customerDao.findByEnuu(enuu);
            return ResultMap.success(customer);
        }
    }

    /**
     * 保存客户的信息
     *
     * @param params 客户的信息
     * @return ResultMap
     */
    @Override
    public ResultMap saveCustomer(String params) {
        if (StringUtils.isEmpty(params)) {
            return new ResultMap(CodeType.FAILURE, "信息缺失");
        } else {
            Customer customer = null;
            try {
                customer = FlexJsonUtils.fromJson(params, Customer.class);
            } catch (Exception e) {
                e.printStackTrace();
                return new ResultMap(CodeType.FAILURE, "信息格式不正确，不能保存");
            }
            ResultMap resultMap = ssoUtil.getSpaceBySpaceName(customer.getCompanyName());
            if (resultMap.getCode() == CodeType.OK.code()) {
                Map<String, Object> content = (Map<String, Object>) resultMap.getContent();
                if (content != null) {
                    Object spaceUU1 = content.get("spaceUU");
                    if (spaceUU1 != null) {
                        Long spaceUU = Long.valueOf(spaceUU1.toString());
                        customer.setEnuu(spaceUU);
                    }
                }
            }
            ResultMap customer1 = validatorCustomer(customer);
            if (customer1.getCode() == CodeType.OK.code()) {
                List<Customer> customers = customerDao.findByWarehousingNo(customer.getWarehousingNo());
                if (!CollectionUtils.isEmpty(customers)) {
                    if (customer.getId() == null) {
                        return new ResultMap(CodeType.FAILURE, "已存在对应的入仓号，请设置其他入仓号");
                    } else {
                        for (Customer customer2 : customers) {
                            if (customer2.getId().longValue() != customer.getId().longValue()) {
                                return new ResultMap(CodeType.FAILURE, "已存在对应的入仓号，请设置其他入仓号");
                            }
                        }
                    }
                }
                if (customer.getId() == null) {
                    customer.setCreateTime(new Date());
                    UserAccount userAccount = SSO.get();
                    customer.setBelongsToEnuu(userAccount.getSpaceUU());
                    customer.setBelongsToCompany(userAccount.getSpaceName());
                }
                try {
                    if (customer.getId() == null) {
                        List<Customer> pCustomers = customerDao.findByEnuuAndBelongsToEnuu(customer.getEnuu(), customer.getBelongsToEnuu());
                        if (!CollectionUtils.isEmpty(pCustomers)) {
                            return new ResultMap(CodeType.FAILURE, "已存在该客户资料信息");
                        }
                    }
                    Customer customer2 = customerDao.save(customer);
                    return ResultMap.success(customer2);
                } catch (Exception e) {
                    return new ResultMap(CodeType.FAILURE, "保存信息失败");
                }
            } else {
                return new ResultMap(CodeType.FAILURE, "客户资料缺失信息");
            }
        }
    }


    /**
     * 验证客户的信息是否正确
     *
     * @param customer 客户信息
     * @return Boolean 验证通过返回Ture，验证失败 false
     */
    @Override
    public ResultMap validatorCustomer(Customer customer) {
        if (customer == null) {
            return new ResultMap(CodeType.FAILURE, "客户资料为空");
        }
        if (StringUtils.isEmpty(customer.getCompanyName())) {
            return new ResultMap(CodeType.FAILURE, "客户公司名称为空");
        }
        if (StringUtils.isEmpty(customer.getTransactionMode())) {
            return new ResultMap(CodeType.FAILURE, "成交方式为空");
        }
        if (StringUtils.isEmpty(customer.getWarehousingNo())) {
            return new ResultMap(CodeType.FAILURE, "委托仓号为空");
        }
        if (customer.getAccountDay() == null) {
            return new ResultMap(CodeType.FAILURE, "结账日信息为空");
        }
        if (customer.getAgencyRate() == null) {
            return new ResultMap(CodeType.FAILURE, "代理费率为空");
        }
        if (customer.getMinAgencyRate() == null) {
            return new ResultMap(CodeType.FAILURE, "最低代理费率为空");
        }
        if (StringUtils.isEmpty(customer.getExchangeRateMethod())) {
            return new ResultMap(CodeType.FAILURE, "汇率方式为空");
        }
        if (StringUtils.isEmpty(customer.getBillMethod())) {
            return new ResultMap(CodeType.FAILURE, "计费方式为空");
        }
        if (StringUtils.isEmpty(customer.getCustomService())) {
            return new ResultMap(CodeType.FAILURE, "客服人员信息为空");
        }
        if (customer.getTaxNum() == null) {
            return new ResultMap(CodeType.FAILURE, "税号信息为空");
        }
        if (StringUtils.isEmpty(customer.getAddress())) {
            return new ResultMap(CodeType.FAILURE, "发票地址信息为空");
        }
        if (StringUtils.isEmpty(customer.getBankAccount())) {
            return new ResultMap(CodeType.FAILURE, "银行账号信息为空");
        }
        if (StringUtils.isEmpty(customer.getCustomscode())) {
            return new ResultMap(CodeType.FAILURE, "海关代码信息为空");
        }
        if (StringUtils.isEmpty(customer.getOrganizationCode())) {
            return new ResultMap(CodeType.FAILURE, "组织机构代码为空");
        }
        if (customer.getAccountPeriodName() == null) {
            return new ResultMap(CodeType.FAILURE, "账期名称为空");
        }
        if (customer.getUnifiedSocialCreditCode() == null) {
            return new ResultMap(CodeType.FAILURE, "统一社会信用代码为空");
        }
        if (StringUtils.isEmpty(customer.getAccountPeriodCode())) {
            return new ResultMap(CodeType.FAILURE, "账期代码为空");
        }
        if (customer.getPayDay() == null) {
            return new ResultMap(CodeType.FAILURE, "付款日为空");
        }
        if (customer.getFixedDay() == null) {
            return new ResultMap(CodeType.FAILURE, "固定日期为空");
        }
        if (customer.getCreditDay() == null) {
            return new ResultMap(CodeType.FAILURE, "信用天数为空");
        }
        return ResultMap.success(null);
    }

    /**
     * 分页获取客资料信息
     *
     * @param params 分页信息
     * @param keyword 需要搜索的关键字
     * @return ResultMap
     */
    @Override
    public ResultMap getCustomer(PageParams params, String keyword) {
        final PageInfo info  = new PageInfo(params);
        info.sorting("createTime", Sort.Direction.DESC);
        info.filter("belongsToEnuu", SSO.get().getSpaceUU());
        if (!StringUtils.isEmpty(keyword)) {
            info.filter("companyName", keyword);
        }
        Page<Customer> customerPage = customerDao.findAll(new Specification<Customer>() {
            @Override
            public Predicate toPredicate(Root<Customer> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
                query.where(info.getPredicates(root, query, cb));
                return null;
            }
        }, info);
        return ResultMap.success(customerPage);
    }

    /**
     * 根据enuu 获取客户信息
     *
     * @param id 客户资料的主键
     * @return
     */
    @Override
    public ResultMap findById(Long id) {
        if (id != null) {
            Customer customer = customerDao.findOne(id);
            return ResultMap.success(customer);
        } else {
            return new ResultMap(CodeType.FAILURE, "客户信息为空");
        }
    }
}
